$error = forceRequiredAddOn("Brick_Treasure_Chest");

if($error == $Error::AddOn_Disabled)
{
	warn("Event_TreasureChestExt - Brick_Treasure_Chest has been enabled by force.");
}

else if($error == $Error::AddOn_NotFound)
{
	error("ERROR: Event_TreasureChestExt - Required add-on Brick_Treasure_Chest not found!");

	return;
}

if(isActivePackage(onChestOpen))
	deactivatePackage(onChestOpen);

if(isActivePackage(Event_onAllChestsFound))
	deactivatePackage(Event_onAllChestsFound);

if(LoadRequiredAddOn("Event_Variables") == $Error::None && isFunction(registerSpecialVar))
{
	registerSpecialVar("GLOBAL", "numChests", "$TreasureChest::NumChests");
	registerSpecialVar(GameConnection, "chestsFound", "%this.numFoundTreasureChests");
	registerSpecialVar(GameConnection, "chestsLeft", "$TreasureChest::NumChests - %this.numFoundTreasureChests");
}

exec("./support.cs");
exec("./support_multiSourceEvents.cs");

registerOutputEvent("fxDTSBrick", "collectChest", "string 32 200" TAB "bool", 1);
registerOutputEvent("fxDTSBrick", "checkChest", "string 32 200", 1);

registerInputEvent("fxDTSBrick", "onChestOpen", "Self fxDTSBrick" TAB "Player Player" TAB "Client GameConnection" TAB "MiniGame MiniGame");
registerInputEvent("fxDTSBrick", "onCheckChestTrue", "Self fxDTSBrick" TAB "Chest fxDTSBrick" TAB "Player Player" TAB "Client GameConnection" TAB "MiniGame MiniGame");
registerInputEvent("fxDTSBrick", "onCheckChestFalse", "Self fxDTSBrick" TAB "Chest fxDTSBrick" TAB "Player Player" TAB "Client GameConnection" TAB "MiniGame MiniGame");
registerMultiSourceInputEvent("fxDTSBrick", "onFoundAllChests", "Self fxDTSBrick" TAB "Player Player" TAB "Client GameConnection" TAB "MiniGame MiniGame", 0);

function fxDTSBrick::collectChest(%brick, %name, %bool, %client)
{
	%name = filterBrickName(%name);

	if(%name !$= "" && isObject("_" @ %name))
		%brick = "_" @ %name;

	%data = %brick.getDataBlock();

	if(!isObject(%client) || !isObject(%player = %client.player) || getTrustLevel(%client, %brick.getGroup()) < 3 || (%data.getID() != brickTreasureChestData.getID() && %data.getID() != brickTreasureChestOpenData.getID()))
		return;

	$InputTarget_["Self"] = %brick;
	$InputTarget_["Player"] = %player;
	$InputTarget_["Client"] = %client;

	if($Server::LAN)
	{
		$InputTarget_["MiniGame"] = getMiniGameFromObject(%client);
	}

	else
	{
		if(getMiniGameFromObject(%brick) == getMiniGameFromObject(%client))
		{
			$InputTarget_["MiniGame"] = getMiniGameFromObject(%brick);
		}

		else
		{
			$InputTarget_["MiniGame"] = 0;
		}
	}

	%hash = sha1(%brick.getPosition() @ %brick.angleID);

	if(!%client.foundTreasureChest_[%hash] && %bool)
	{
		%brick.processInputEvent("onChestOpen", %client);

		%brick.openTreasureChest(%player);
	}

	else if(%client.foundTreasureChest_[%hash] && !%bool)
	{
		%client.removeTreasureChestHash(%hash);

		if(%client.numFoundTreasureChests >= $TreasureChest::NumChests)
		{
			if($TreasureChest::NumChests == 1)
			{
				commandToClient(%client, 'BottomPrint', "\c3You have lost the treasure chest!", 2);
			}

			else
			{
				commandToClient(%client, 'BottomPrint', "\c3You have found all" SPC $TreasureChest::NumChests SPC "treasure chests!", 2);

				%elapsedtime = getSimTime() - %client.treasureVictoryTime;

				if(%elapsedTime > 10000)
				{
					%client.treasureVictoryTime = getSimTime();

					messageAll('', "\c3" @ %client.getPlayerName() SPC "\c0found all\c6" SPC $TreasureChest::NumChests SPC "\c0treasure chests!");
					commandToClient(%client, 'BottomPrint', "\c3You have found all" SPC $TreasureChest::NumChests SPC "treasure chests!", 2);
				}
			}
		}

		else
		{
			commandToClient(%client, 'BottomPrint', "\c3You have found" SPC %client.numFoundTreasureChests SPC "/" SPC $TreasureChest::NumChests SPC "treasure chests! You lost 1 treasure chest!", 2);
		}
	}
}

function fxDTSBrick::checkChest(%brick, %name, %client)
{
	%name = filterBrickName(%name);

	if(%name !$= "" && isObject("_" @ %name))
	{
		%originBrick = %brick;
		%brick = "_" @ %name;
	}

	%data = %brick.getDataBlock();

	if(!isObject(%client) || getTrustLevel(%client, %brick.getGroup()) < 3 || (%data.getID() != brickTreasureChestData.getID() && %data.getID() != brickTreasureChestOpenData.getID()))
		return;

	%hash = sha1(%brick.getPosition() @ %brick.angleID);

	$InputTarget_["Self"] = %originBrick;
	$InputTarget_["Chest"] = %brick;
	$InputTarget_["Player"] = %client.player;
	$InputTarget_["Client"] = %client;

	if($Server::LAN)
	{
		$InputTarget_["MiniGame"] = getMiniGameFromObject(%client);
	}

	else
	{
		if(getMiniGameFromObject(%brick) == getMiniGameFromObject(%client))
		{
			$InputTarget_["MiniGame"] = getMiniGameFromObject(%brick);
		}

		else
		{
			$InputTarget_["MiniGame"] = 0;
		}
	}

	if(isObject(%originBrick))
		%brick = %originBrick;

	if(%client.foundTreasureChest_[%hash])
		%brick.processInputEvent("onCheckChestTrue", %client);
	else
		%brick.processInputEvent("onCheckChestFalse", %client);
}

package TreasureChestExt
{
	function fxDTSBrick::onActivate(%brick, %player, %client, %pos, %vec)
	{  
		%data = %brick.getDataBlock();

		if(%data.getID() == brickTreasureChestData.getID())
		{
			%hash = sha1(%brick.getPosition() @ %brick.angleID);

			if(!%client.foundTreasureChest_[%hash])
			{
				$InputTarget_["Self"] = %brick;
				$InputTarget_["Player"] = %player;
				$InputTarget_["Client"] = %client;

				if($Server::LAN)
				{
					$InputTarget_["MiniGame"] = getMiniGameFromObject(%client);
				}

				else
				{
					if(getMiniGameFromObject(%brick) == getMiniGameFromObject(%client))
					{
						$InputTarget_["MiniGame"] = getMiniGameFromObject(%brick);
					}

					else
					{
						$InputTarget_["MiniGame"] = 0;
					}
				}

				%brick.processInputEvent("onChestOpen", %client);

				%brick.openTreasureChest(%player);
			}

			else
			{
				parent::onActivate(%brick, %player, %client, %pos, %vec);

				%brick.processInputEvent(onActivate, %client);
			}
		}

		else if(%data.getID() == brickTreasureChestOpenData.getID())
		{
			//Don't do onActivate event.
		}

		else
		{
			parent::onActivate(%brick, %player, %client, %pos, %vec);
		}
	}

	function brickTreasureChestData::onPlant(%data, %brick)
	{
		parent::onPlant(%data, %brick);

		//We want the sound to play onChestOpen instead of onActivate.

		%brick.clearEvents();

		%enabled     = 1;
		%delay       = 0;
		%inputEvent  = "onChestOpen";
		%target      = "Client";
		%outputEvent = "PlaySound";
		%par1        = rewardSound.getID();

		%brick.addEvent(%enabled, %delay, %inputEvent, %target, %outputEvent, %par1);
	}

	function brickTreasureChestData::openTreasureChest(%data, %brick, %player)
	{
		if(getSimTime() - %client.treasureVictoryTime <= 10000)
			%spam = true;

		parent::openTreasureChest(%data, %brick, %player);

		if(!isObject(%client = %player.client) || %client.numFoundTreasureChests < $TreasureChest::NumChests || %spam)
			return;

		$InputTarget_["Self"] = %brick;
		$InputTarget_["Player"] = %player;
		$InputTarget_["Client"] = %client;
		$InputTarget_["MiniGame"] = %mini = getMiniGameFromObject(%brick);

		processMultiSourceInputEvent("onFoundAllChests", %client, %mini);
	}

	function serverCmdAddEvent(%client, %enabled, %input, %delay, %target, %a, %output, %par1, %par2, %par3, %par4)
	{
		%targetList = $InputEvent_TargetList["fxDTSBrick", inputEvent_getInputEventIdx("fxDTSBrick", %input)];

		if((getWord(getField(%targetList, %target), 1) $= "fxDTSBrick" || %target == -1) && (%output == outputEvent_getOutputEventIdx("fxDTSBrick", "collectChest") || %output == outputEvent_getOutputEventIdx("fxDTSBrick", "checkChest")))
			%par1 = filterBrickName(%par1);

		return parent::serverCmdAddEvent(%client, %enabled, %input, %delay, %target, %a, %output, %par1, %par2, %par3, %par4);
	}
};
activatePackage(TreasureChestExt);